@@ -14,12 +14,7 @@
 
 import unittest
 
-import os
-import sys
-_test_dir = os.path.dirname(os.path.abspath(__file__))
-_code_dir = os.path.dirname(_test_dir)
-if not _code_dir in sys.path:
-    sys.path.insert(1, _code_dir)
+import tools
 
 import enum
 
@@ -57,15 +52,14 @@ class ValuesFixture(object):
         )
         planet_keys = self.planet_dict.keys()
 
-        colour_keys = (
+        colour_keys = [
             'red', 'green', 'blue',
             'yellow', 'orange', 'purple',
             'white', 'black',
-        )
+        ]
 
-        self.SimpleEnum = self.enum_factory_class(
-            *('spam', 'eggs', 'beans')
-        )
+        simple_keys = ['spam', 'eggs', 'beans']
+        self.SimpleEnum = self.enum_factory_class(*simple_keys)
 
         Colour = self.enum_factory_class(*colour_keys)
         Planet = self.enum_factory_class(*planet_keys)
@@ -79,10 +73,11 @@ class ValuesFixture(object):
         }
 
         for enumtype, params in self.valid_values.items():
+            params['enumtype'] = enumtype
             values = {}
             for i, key in enumerate(params['keys']):
                 values[key] = enum.EnumValue(enumtype, i, key)
-            params.update(dict(enumtype=enumtype, values=values))
+            params['values'] = values
 
 
 class Test_Module(unittest.TestCase):
@@ -187,12 +182,9 @@ class Test_EnumException(unittest.TestCase):
             instance = exc_type(*args)
             self.valid_exceptions[exc_type]['instance'] = instance
 
-    def iterate_test(self, test_func, params_dict=None):
-        """ Iterate a test over a set of parameters """
-        if not params_dict:
-            params_dict = self.valid_exceptions
-        for exc_type, params in params_dict.items():
-            test_func(exc_type, params)
+        self.iterate_params = tools.make_params_iterator(
+            default_params_dict = self.valid_exceptions
+        )
 
     def test_EnumException_abstract(self):
         """ The module exception base class should be abstract """
@@ -202,24 +194,21 @@ class Test_EnumException(unittest.TestCase):
 
     def test_exception_instance(self):
         """ Exception instance should be created """
-        def test_func(exc_type, params):
-            self.failUnless(params['instance'])
-
-        self.iterate_test(test_func)
+        for exc_type, params in self.iterate_params():
+            instance = params['instance']
+            self.failUnless(instance)
 
     def test_exception_types(self):
         """ Exception instances should match expected types """
-        def test_func(exc_type, params):
+        for exc_type, params in self.iterate_params():
+            instance = params['instance']
             for match_type in params['types']:
-                instance = params['instance']
                 self.failUnless(isinstance(instance, match_type),
                     msg = "instance: %s, match_type: %s" % (
                         repr(instance), match_type
                     )
                 )
 
-        self.iterate_test(test_func)
-
 
 class Test_EnumValue(unittest.TestCase, ValuesFixture):
     """ Test case for the EnumValue class """
@@ -229,99 +218,95 @@ class Test_EnumValue(unittest.TestCase, ValuesFixture):
         self.enum_factory_class = Mock_Enum
         ValuesFixture.setUp(self)
 
-    def iterate_test(self, test_func, datasets=None):
-        """ Iterate a test function for sets of values """
-        if not datasets:
-            datasets = self.valid_values.keys()
-        for params in [self.valid_values[s] for s in datasets]:
-            for i, key in enumerate(params['keys']):
-                enumtype = params['enumtype']
-                value = params['values'][key]
-                test_func(enumtype, i, key, value)
+        self.iterate_params = tools.make_params_iterator(
+            default_params_dict = self.valid_values
+        )
 
     def test_instantiate(self):
         """ Creating an EnumValue instance should succeed """
-        def test_func(enumtype, i, key, value):
-            self.failUnless(value)
-
-        self.iterate_test(test_func)
+        for enumtype, params in self.iterate_params():
+            for key, instance in params['values'].items():
+                self.failUnless(instance)
 
     def test_enumtype_equal(self):
         """ EnumValue should export its enum type """
-        def test_func(enumtype, i, key, value):
-            self.failUnlessEqual(enumtype, value.enumtype)
-
-        self.iterate_test(test_func)
+        for enumtype, params in self.iterate_params():
+            for key, instance in params['values'].items():
+                self.failUnlessEqual(enumtype, instance.enumtype)
 
     def test_key_equal(self):
         """ EnumValue should export its string key """
-        def test_func(enumtype, i, key, value):
-            self.failUnlessEqual(key, value.key)
-
-        self.iterate_test(test_func)
+        for enumtype, params in self.iterate_params():
+            for key, instance in params['values'].items():
+                self.failUnlessEqual(key, instance.key)
 
     def test_str_key(self):
         """ String value for EnumValue should be its key string """
-        def test_func(enumtype, i, key, value):
-            self.failUnlessEqual(key, str(value))
-
-        self.iterate_test(test_func)
+        for enumtype, params in self.iterate_params():
+            for key, instance in params['values'].items():
+                self.failUnlessEqual(key, str(instance))
 
     def test_index_equal(self):
         """ EnumValue should export its sequence index """
-        def test_func(enumtype, i, key, value):
-            self.failUnlessEqual(i, value.index)
-
-        self.iterate_test(test_func)
+        for enumtype, params in self.iterate_params():
+            for i, key in enumerate(params['keys']):
+                instance = params['values'][key]
+                self.failUnlessEqual(i, instance.index)
 
     def test_repr(self):
         """ Representation of EnumValue should be meaningful """
-        def test_func(enumtype, i, key, value):
-            value_repr = repr(value)
-            self.failUnless(value_repr.startswith('EnumValue('))
-            self.failUnless(value_repr.count(repr(enumtype)))
-            self.failUnless(value_repr.count(repr(i)))
-            self.failUnless(value_repr.count(repr(key)))
-            self.failUnless(value_repr.endswith(')'))
+        for enumtype, params in self.iterate_params():
+            for i, key in enumerate(params['keys']):
+                instance = params['values'][key]
+                repr_str = repr(instance)
+                self.failUnless(repr_str.startswith('EnumValue('))
+                self.failUnless(repr_str.count(repr(enumtype)))
+                self.failUnless(repr_str.count(repr(i)))
+                self.failUnless(repr_str.count(repr(key)))
+                self.failUnless(repr_str.endswith(')'))
 
     def test_hash_equal(self):
         """ Each EnumValue instance should have same hash as its value """
-        def test_func(enumtype, i, key, value):
-            self.failUnlessEqual(hash(i), hash(value))
+        for enumtype, params in self.iterate_params():
+            for i, key in enumerate(params['keys']):
+                instance = params['values'][key]
+                self.failUnlessEqual(hash(i), hash(instance))
 
     def test_hash_unequal(self):
         """ Different EnumValue instances should have different hashes """
-        def test_func(enumtype, i, key, value):
-            dataset = self.valid_values[enumtype]
-            for j, other_key in enumerate(dataset['keys']):
-                if i == j:
-                    continue
-                other_value = dataset['values'][other_key]
-                self.failIfEqual(hash(value), hash(other_value))
-
-        self.iterate_test(test_func)
+        for enumtype, params in self.iterate_params():
+            for i, key in enumerate(params['keys']):
+                instance = params['values'][key]
+                for j, other_key in enumerate(params['keys']):
+                    if i == j:
+                        continue
+                    other_instance = params['values'][other_key]
+                    self.failIfEqual(
+                        hash(instance),
+                        hash(other_instance)
+                    )
 
     def test_cmp_equal(self):
         """ An EnumValue should compare equal to its value """
-        def test_func(enumtype, i, key, value):
-            self.failUnlessEqual(value,
-                enum.EnumValue(enumtype, i, key)
-            )
-
-        self.iterate_test(test_func)
+        for enumtype, params in self.iterate_params():
+            for i, key in enumerate(params['keys']):
+                instance = params['values'][key]
+                self.failUnlessEqual(instance,
+                    enum.EnumValue(enumtype, i, key)
+                )
 
     def test_cmp_unequal(self):
         """ An EnumValue should compare different to other values """
-        def test_func(enumtype, i, key, value):
-            self.failIfEqual(value,
-                enum.EnumValue(enumtype, None, None)
-            )
-
-        self.iterate_test(test_func)
+        for enumtype, params in self.iterate_params():
+            for i, key in enumerate(params['keys']):
+                instance = params['values'][key]
+                self.failIfEqual(instance,
+                    enum.EnumValue(enumtype, None, None)
+                )
 
     def test_cmp_sequence(self):
         """ EnumValue instances should compare as their sequence order """
-        for enumtype, params in self.valid_values.items():
+        for enumtype, params in self.iterate_params():
             for i, left_key in enumerate(params['keys']):
                 for j, right_key in enumerate(params['keys']):
                     self.failUnlessEqual(cmp(i, j),
@@ -330,36 +315,55 @@ class Test_EnumValue(unittest.TestCase, ValuesFixture):
                         )
                     )
 
-    def test_cmp_unrelated_enum(self):
-        """ An EnumValue should not allow comparison to other enums """
-        def test_func(enumtype, i, key, value):
-            self.failUnlessRaises(enum.EnumValueCompareError,
-                cmp, value, enum.EnumValue(self.SimpleEnum, i, key)
-            )
-
-        self.iterate_test(test_func)
+    def test_cmp_different_enum(self):
+        """ An EnumValue should not implement comparison to other enums """
+        for enumtype, params in self.iterate_params():
+            for i, key in enumerate(params['keys']):
+                instance = params['values'][key]
+                test_value = enum.EnumValue(self.SimpleEnum, i, key)
+                compare_result = instance.__cmp__(test_value)
+                self.failUnlessEqual(NotImplemented, compare_result)
 
     def test_cmp_non_enum(self):
-        """ An EnumValue should not allow comparison to other types """
+        """ An EnumValue should not implement comparison to other types """
         test_value = enum.EnumValue(self.SimpleEnum, 0, 'test')
-        for other in self.other_values:
-            self.failUnlessRaises(enum.EnumValueCompareError,
-                cmp, test_value, other
-            )
+        for other_value in self.other_values:
+            compare_result = test_value.__cmp__(other_value)
+            self.failUnlessEqual(NotImplemented, compare_result)
+
+    def test_compare_equality_different_enum(self):
+        """ An EnumValue should compare inequal to values of other enums """
+        for enumtype, params in self.iterate_params():
+            for i, key in enumerate(params['keys']):
+                instance = params['values'][key]
+                test_value = enum.EnumValue(self.SimpleEnum, i, key)
+                self.failIfEqual(test_value, instance)
+
+    def test_compare_equality_non_enum(self):
+        """ An EnumValue should compare inequal to any other value """
+        test_value = enum.EnumValue(self.SimpleEnum, 0, 'test')
+        for other_value in self.other_values:
+            self.failIfEqual(test_value, other_value)
+
+    def test_sequence_other_values(self):
+        """ An EnumValue should compare sequentially to other values """
+        test_value = enum.EnumValue(self.SimpleEnum, 0, 'test')
+        test_list = list(self.other_values)
+        test_list.append(test_value)
+        test_list.sort()
+        self.failUnless(test_value in test_list)
 
     def test_value_key(self):
         """ An EnumValue should have the specified key """
-        def test_func(enumtype, i, key, instance):
-            self.failUnlessEqual(key, instance.key)
-
-        self.iterate_test(test_func)
+        for enumtype, params in self.iterate_params():
+            for key, instance in params['values'].items():
+                self.failUnlessEqual(key, instance.key)
 
     def test_value_enumtype(self):
         """ An EnumValue should have its associated enumtype """
-        def test_func(enumtype, i, key, instance):
-            self.failUnlessEqual(enumtype, instance.enumtype)
-
-        self.iterate_test(test_func)
+        for enumtype, params in self.iterate_params():
+            for key, instance in params['values'].items():
+                self.failUnlessEqual(enumtype, instance.enumtype)
 
 
 class Test_Enum(unittest.TestCase, ValuesFixture):
@@ -370,13 +374,9 @@ class Test_Enum(unittest.TestCase, ValuesFixture):
         self.enum_factory_class = enum.Enum
         ValuesFixture.setUp(self)
 
-    def iterate_test(self, test_func, datasets=None):
-        """ Iterate a test function for sets of values """
-        if not datasets:
-            datasets = self.valid_values.keys()
-        for enumtype, params in [(s, self.valid_values[s])
-                                 for s in datasets]:
-            test_func(enumtype, params)
+        self.iterate_params = tools.make_params_iterator(
+            default_params_dict = self.valid_values
+        )
 
     def test_empty_enum(self):
         """ Enum constructor should refuse empty keys sequence """
@@ -394,133 +394,108 @@ class Test_Enum(unittest.TestCase, ValuesFixture):
 
     def test_value_attributes(self):
         """ Enumeration should have attributes for each value """
-        def test_func(enumtype, params):
+        for enumtype, params in self.iterate_params():
             for i, key in enumerate(params['keys']):
-                value = getattr(enumtype, key)
+                instance = getattr(enumtype, key)
                 test_value = enum.EnumValue(enumtype, i, key)
-                self.failUnlessEqual(test_value, value)
-
-        self.iterate_test(test_func)
+                self.failUnlessEqual(test_value, instance)
 
     def test_length(self):
         """ Enumeration should have length of its value set """
-        def test_func(enumtype, params):
+        for enumtype, params in self.iterate_params():
             self.failUnlessEqual(len(params['values']), len(enumtype))
 
-        self.iterate_test(test_func)
-
     def test_value_items(self):
         """ Enumeration should have items for each value """
-        def test_func(enumtype, params):
+        for enumtype, params in self.iterate_params():
             for i, key in enumerate(params['keys']):
                 value = enumtype[i]
                 test_value = enum.EnumValue(enumtype, i, key)
                 self.failUnlessEqual(test_value, value)
 
-        self.iterate_test(test_func)
-
     def test_iterable(self):
         """ Enumeration class should iterate over its values """
-        def test_func(enumtype, params):
+        for enumtype, params in self.iterate_params():
             for i, value in enumerate(enumtype):
                 key = params['keys'][i]
                 test_value = params['values'][key]
             self.failUnlessEqual(value, test_value)
 
-        self.iterate_test(test_func)
-
     def test_iterate_sequence(self):
         """ Enumeration iteration should match specified sequence """
-        def test_func(enumtype, params):
+        for enumtype, params in self.iterate_params():
             values_dict = params['values']
             values_seq = [values_dict[key] for key in params['keys']]
             enum_seq = [val for val in enumtype]
             self.failUnlessEqual(values_seq, enum_seq)
             self.failIfEqual(values_seq.reverse(), enum_seq)
 
-        self.iterate_test(test_func)
-
     def test_membership_bogus(self):
         """ Enumeration should not contain bogus values """
-        for value in self.other_values:
-            self.failIf(value in self.SimpleEnum)
+        for enumtype, params in self.iterate_params():
+            for value in self.other_values:
+                self.failIf(value in enumtype)
 
     def test_membership_value(self):
         """ Enumeration should contain explicit value """
-        def test_func(enumtype, params):
+        for enumtype, params in self.iterate_params():
             for i, key in enumerate(params['keys']):
                 value = params['values'][key]
                 self.failUnless(value in enumtype)
 
-        self.iterate_test(test_func)
-
     def test_membership_key(self):
         """ Enumeration should contain key string """
-        def test_func(enumtype, params):
+        for enumtype, params in self.iterate_params():
             for key in params['keys']:
                 self.failUnless(key in enumtype)
 
-        self.iterate_test(test_func)
-
     def test_add_attribute(self):
         """ Enumeration should refuse attribute addition """
-        def test_func(enumtype, params):
+        for enumtype, params in self.iterate_params():
             self.failUnlessRaises(enum.EnumImmutableError,
                 setattr, enumtype, 'bogus', "bogus"
             )
 
-        self.iterate_test(test_func)
-
     def test_modify_attribute(self):
         """ Enumeration should refuse attribute modification """
-        def test_func(enumtype, params):
+        for enumtype, params in self.iterate_params():
             for key in params['keys']:
                 self.failUnlessRaises(enum.EnumImmutableError,
                     setattr, enumtype, key, "bogus"
                 )
 
-        self.iterate_test(test_func)
-
     def test_delete_attribute(self):
         """ Enumeration should refuse attribute deletion """
-        def test_func(enumtype, params):
+        for enumtype, params in self.iterate_params():
             for key in params['keys']:
                 self.failUnlessRaises(enum.EnumImmutableError,
                     delattr, enumtype, key
                 )
 
-        self.iterate_test(test_func)
-
     def test_add_item(self):
         """ Enumeration should refuse item addition """
-        def test_func(enumtype, params):
+        for enumtype, params in self.iterate_params():
             index = len(params['keys'])
             self.failUnlessRaises(enum.EnumImmutableError,
                 enumtype.__setitem__, index, "bogus"
             )
 
-        self.iterate_test(test_func)
-
     def test_modify_item(self):
         """ Enumeration should refuse item modification """
-        def test_func(enumtype, params):
+        for enumtype, params in self.iterate_params():
             for i, key in enumerate(params['keys']):
                 self.failUnlessRaises(enum.EnumImmutableError,
                     enumtype.__setitem__, i, "bogus"
                 )
 
-        self.iterate_test(test_func)
-
     def test_delete_item(self):
         """ Enumeration should refuse item deletion """
-        def test_func(enumtype, params):
+        for enumtype, params in self.iterate_params():
             for i, key in enumerate(params['keys']):
                 self.failUnlessRaises(enum.EnumImmutableError,
                     enumtype.__delitem__, i
                 )
 
-        self.iterate_test(test_func)
-
 
 def suite():
     """ Create the test suite for this module """
@@ -545,5 +520,6 @@ def __main__(argv=None):
     return exitcode
 
 if __name__ == '__main__':
+    import sys
     exitcode = __main__(sys.argv)
     sys.exit(exitcode)
